from __future__ import print_function, division
import numpy as np
from Bio import Phylo
import matplotlib.pyplot as plt
from treetime import TreeAnc
import treetime

if __name__ == '__main__':
    plt.ion()

    # path to data
    base_name = '../data/h3n2_na/h3n2_na_200'

    # instantiate treetime
    myTree = TreeAnc(gtr='Jukes-Cantor', tree=base_name+".nwk", aln=base_name+'.fasta', verbose = 0)

    # infer a GTR model and ML ancestral sequences, either marginally or jointly most likely
    myTree.infer_ancestral_sequences(infer_gtr=True, marginal=True)

    # lets examine the properties of a node in the tree after ancestral inference
    node = myTree.tree.get_nonterminals()[7]
    # each node now has an inferred sequence
    if treetime.version<"0.7":
        print("\nthe inferred sequences is an array of characters:", node.sequence)
    else:
        print("\nthe inferred sequences is an array of characters:", myTree.sequence(node, as_string=False))

    # in addition, each node of the tree now has an mutation object attached
    # note that the mutation numbering starts at 0 rather than 1
    print("\nmutations on the branch leading to node %s:"%node.name, node.mutations)

    # we can readily verify these mutations by checking the inferred sequences
    if node.mutations:
        mut = node.mutations[0]
        if treetime.version<"0.7":
            print("\nmutation %s%d%s corresponds to"%mut,
                  "parent state: %s, child state %s\n\n"%(node.up.sequence[mut[1]], node.sequence[mut[1]]))
        else:
            print("\nmutation %s%d%s corresponds to"%mut,
                  "parent state: %s, child state %s\n\n"%(myTree.sequence(node.up)[mut[1]], myTree.sequence(node)[mut[1]]))


    # plot the tree and label branches by mutation
    fig, ax = plt.subplots(1,1,figsize=(18,13))
    ax.set_title("branches annotated by inferred mutations", fontsize=18)
    ax.set_axis_off()
    Phylo.draw(myTree.tree, label_func = lambda x:"", axes=ax,
        branch_labels=lambda x:", ".join(map(lambda m: "%s%d%s"%m, x.mutations[:3]))
                               +('...' if len(x.mutations)>3 else ""))

    # finally, we print the inferred GTR model
    print(myTree.gtr)



from __future__ import print_function, division
import numpy as np
from Bio import Phylo
import matplotlib.pyplot as plt
from matplotlib import cm
try:
    import seaborn as sns
    sns.set_style('whitegrid')
except:
    print ("Seaborn not found. Default style will be used for the plots")

from treetime import TreeTime
from treetime.utils import parse_dates

def format_axes(fig, axs):
    axs[0].set_axis_off()
    axs[1].tick_params(labelsize=14)
    axs[1].set_ylabel("root-to-tip distance", fontsize=16)
    axs[1].set_xlabel("date", fontsize=16)
    fig.tight_layout()


if __name__ == '__main__':

    # load data and parse dates
    plt.ion()
    base_name = '../data/h3n2_na/h3n2_na_20'

    dates = parse_dates(base_name+'.metadata.csv')
    tt = TreeTime(gtr='Jukes-Cantor', tree = base_name+'.nwk',
                  aln = base_name+'.fasta', verbose = 1, dates = dates)

    # inititally the root if the tree is a mess:
    fig, axs = plt.subplots(1,2, figsize=(18,9))
    axs[0].set_title("Arbitrarily rooted tree", fontsize=18)
    axs[1].set_title("Inverse divergence-time relationship", fontsize=18)
    Phylo.draw(tt.tree, show_confidence=False, axes=axs[0], label_func=lambda x:x.name.split('|')[0] if x.is_terminal() else "")
    tt.plot_root_to_tip(ax=axs[-1])
    format_axes(fig, axs)

    # lets reroot: we now have a positve correlation of root-to-tip distance with sampling date
    tt.reroot(root="least-squares")
    fig, axs = plt.subplots(1,2, figsize=(18,9))
    axs[0].set_title("Tree rerooted by treetime", fontsize=18)
    axs[1].set_title("Optimal divergence-time relationship", fontsize=18)
    Phylo.draw(tt.tree, show_confidence=False, axes=axs[0],
               label_func=lambda x:x.name.split('|')[0] if x.is_terminal() else "")
    tt.plot_root_to_tip(ax=axs[-1])
    format_axes(fig, axs)

    # rerooting can be done along with the tree time inference
    tt.run(root="best", branch_length_mode='input', max_iter=2)
    # if more complicated models (relaxed clocks, coalescent models) are to be used
    # or you want to resolve polytomies, treetime needs to be run for
    # several iterations, for example as
    # tt.run(root="best", resolve_polytomies=True, max_iter=2)

    # each node is now at a position that correspond to the given or inferred date
    # the units of branch length are still clock rate.
    print("clock rate: %1.5f"%tt.date2dist.clock_rate)
    fig, axs = plt.subplots(1,2, figsize=(18,9))
    Phylo.draw(tt.tree, label_func=lambda x:'', show_confidence=False, axes=axs[0])
    axs[0].set_title("Tree: units are substitutions", fontsize=18)

    # we can convert the branch length to units in years and redraw
    tt.branch_length_to_years()
    Phylo.draw(tt.tree, label_func=lambda x:'', show_confidence=False, axes=axs[1])
    axs[1].set_title("Tree: units are years", fontsize=18)
    axs[0].tick_params(labelsize=14)
    axs[1].tick_params(labelsize=14)
    fig.tight_layout()

    # treetime implements a convenience function to plot timetrees
    from treetime.treetime import plot_vs_years
    plot_vs_years(tt, label_func=lambda x:"", show_confidence=False)




'''
this script illustrates the use of treetime to analyze viral sequences.
As an example, we use Ebola virus sequences from the 2014-2015 outbreak in
West Africa. This example contains more than 300 sequences, it will take
a few minutes to run.
'''

from __future__ import print_function, division
import sys
import numpy as np
from Bio import Phylo

import matplotlib.pyplot as plt
from matplotlib import cm
try:
    import seaborn as sns
    sns.set_style('whitegrid')
except:
    print ("Seaborn not found. Default style will be used for the plots")

from treetime import TreeTime
from treetime.utils import parse_dates
import treetime.config as ttconf

if __name__ == '__main__':


    plt.ion()
    base_name = '../data/ebola/ebola'
    dates = parse_dates(base_name+'.metadata.csv')
    # instantiate treetime
    ebola = TreeTime(gtr='Jukes-Cantor', tree = base_name+'.nwk', precision=1,
                     aln = base_name+'.fasta', verbose = 2, dates = dates)

    # infer an ebola time tree while rerooting and resolving polytomies
    res = ebola.run(root='best', infer_gtr=True, relaxed_clock=False, max_iter=2,
                    branch_length_mode='input', n_iqd=3, resolve_polytomies=True,
                    Tc='skyline', time_marginal="assign", vary_rate=True)

    if res==ttconf.ERROR:
        sys.exit()

    # scatter root to tip divergence vs sampling date
    ebola.plot_root_to_tip(add_internal=True)
    plt.legend(loc=2)

    # rescale branch length to years and plot in axis 0
    from treetime.treetime import plot_vs_years
    fig, axs = plt.subplots(1,2, sharey=True, figsize=(12,8))
    plot_vs_years(ebola, step=1, ax=axs[1], confidence=(0.05,0.95), label_func = lambda x:"")
    axs[1].set_xlim(0, 2.5)
    axs[1].set_title("time tree")

    # assign mutation length to branch length and plot the mutation tree in axs[1]
    axs[0].set_title("mutation tree")
    for n in ebola.tree.find_clades():
        n.branch_length=n.mutation_length
    Phylo.draw(ebola.tree, label_func=lambda x:"", axes=axs[0])
    plt.tight_layout()

    # reset branch length to time (in substitution rate units)
    for n in ebola.tree.find_clades():
        n.branch_length=n.clock_length

    # OUTPUT the GTR model
    print(ebola.gtr)

    # plot Skyline
    skyline, confidence = ebola.merger_model.skyline_inferred(gen=50, confidence=2.0)
    skyline_empirical = ebola.merger_model.skyline_empirical(gen=50)
    plt.figure()
    plt.fill_between(skyline.x, confidence[0], confidence[1], color=(0.8, 0.8, 0.8))
    plt.plot(skyline.x, skyline.y, label='maximum likelihood skyline')
    plt.plot(skyline_empirical.x, skyline_empirical.y, label='empirical skyline')
    plt.yscale('log')
    plt.legend()
    plt.ticklabel_format(axis='x',useOffset=False)








"""
This script illustrates the use of the relaxed clock model in TreeTime.
TreeTime currently only supports a normally distributed clock model.
"""

from __future__ import print_function, division
import sys
import numpy as np
from Bio import Phylo

import matplotlib.pyplot as plt
from matplotlib import cm
try:
    import seaborn as sns
    sns.set_style('whitegrid')
except:
    print ("Seaborn not found. Default style will be used for the plots")

from treetime import TreeTime
from treetime.utils import parse_dates
import treetime.config as ttconf


if __name__ == '__main__':

    # load data and parse dates
    plt.ion()
    base_name = '../data/h3n2_na/h3n2_na_20'
    dates = parse_dates(base_name+'.metadata.csv')

    # instantiate treetime
    tt_relaxed = TreeTime(gtr='Jukes-Cantor', tree = base_name+'.nwk',
                        aln = base_name+'.fasta', verbose = 4, dates = dates)

    # this example uses an autocorrelated molecular clock with normal prior and parent-child coupling
    # the parameter slack penalizes rate deviations from the average rate
    # couplings penalize rate changes between parent and child nodes.
    tt_relaxed.run(root='best', relaxed_clock={"slack":5.0, "coupling":1.0}, max_iter=3,
                   resolve_polytomies=True, Tc=0, time_marginal=False)



    ##############
    # PLOTTING
    ##############

    # draw trees inferred with the relaxed model
    fig = plt.figure()
    ax = plt.subplot(111)
    vmin, vmax = 0.5, 1.5 # color branches according to the rate deviation
    for n in tt_relaxed.tree.find_clades():
        if n.up:
            n.color = [int(x*255) for x in cm.cool((min(max(vmin, n.branch_length_interpolator.gamma),vmax)-vmin)/(vmax-vmin))[:3]]
        else:
            n.color = [200,200,200]

    ax.set_title("relaxed clock")
    Phylo.draw(tt_relaxed.tree, axes=ax, show_confidence=False, label_func = lambda x:'')

    # Scatter branch stretch against the rate multiplier of the branch.
    # they are expected to have a positive correlation
    # 1) collect the optimal branch lenght (called mutation_length),
    #    the timetree branch length (clock_length) and the inferred rate multiplier gamma
    branch_lengths = []
    for n in tt_relaxed.tree.find_clades():
        if n.up:
            branch_lengths.append((n.mutation_length, n.clock_length, n.branch_length_interpolator.gamma))


    # 2) plot the difference between optimal and timetree branch length vs the rate multiplier
    plt.figure()
    branch_lengths = np.array(branch_lengths)
    plt.scatter(branch_lengths[:,0]-branch_lengths[:,1], branch_lengths[:,2])
    plt.xlabel("stretch")
    plt.ylabel("rate multiplier")


